#include "dialogs.h"
#include "engine.h"
#include "isocanvas.h"
#include "maindlg.h"

using namespace MAS;

void TrainDialog::HandleEvent(Widget &obj, int msg, int arg1, int arg2) 
{
	Window::HandleEvent (obj, msg, arg1, arg2);
}

StationDialog::StationDialog() : XmlDialog(), idStation(-1)
{
	readFromXml (resources.getDialog ("station"));		

	myTimer = Timer::Install(500);	
}

void StationDialog::setStation (int s) 
{ 
	idStation = s; 
	update();
}

void StationDialog::HandleEvent(Widget &obj, int msg, int arg1, int arg2) 
{
	Window::HandleEvent (obj, msg, arg1, arg2);
	
	switch (msg)
	{
		case 1003: {
				assert (idStation >= 0);
				int result = engine.addTrain (idStation);				
				if (result == -2)
				{
					MessageBox msg("No Money", "You don't have money for that", NULL, NULL, "OK");
					msg.Popup (this);
				}
				else if (result == -1)
				{
					MessageBox msg("Error", "First build two stations and a railway between them!", NULL, NULL, "OK");
					msg.Popup (this);
				}
				else
				{
					isoCanvas->Redraw();
					
					// popup train dialog
					
					MainDlg *mainDlg = dynamic_cast<MainDlg*>(isoCanvas->GetParent());
					assert (mainDlg);
					
					TrainDialog *td = new TrainDialog();
					td->setTrain(result);
					mainDlg->Add (*td);
				}
				
			}
			break;
	}
}

void TimeTableDialog::HandleEvent(Widget &obj, int msg, int arg1, int arg2) 
{
	Window::HandleEvent (obj, msg, arg1, arg2);
	
	switch (msg)
	{
		case 1000: {
				Close();
			}
			break;
	}
}

TrainDialog::TrainDialog() : XmlDialog(), idTrain(-1)
{
	readFromXml (resources.getDialog ("train"));
	myTimer = Timer::Install(500);	
	
}

void TrainDialog::setTrain (int s)
{
	idTrain = s;
	
	assert (idTrain >= 0);
	update();	
}

void StationDialog::update()
{
	if (idStation >= 0)
	{
		assert (widgets.find (1002) != widgets.end());	
		Label *lb = dynamic_cast <Label *> (widgets[1002]);	
		assert (lb != NULL);
			
		lb->SetText (engine.station[idStation].name.c_str());
	}
}

void TrainDialog::update()
{
	Label *lbl;
	assert (widgets.find (1102) != widgets.end());	
	lbl = dynamic_cast <Label *> (widgets[1102]);	
	assert (lbl != NULL);
		
	lbl->SetNumber (idTrain);

	assert (widgets.find (1104) != widgets.end());	
	lbl = dynamic_cast <Label *> (widgets[1104]);	
	assert (lbl != NULL);
	
	lbl->SetText (engine.station[engine.train[idTrain].program[0]].name.c_str());

	assert (widgets.find (1106) != widgets.end());	
	lbl = dynamic_cast <Label *> (widgets[1106]);	
	assert (lbl != NULL);
	
	lbl->SetText (engine.station[engine.train[idTrain].program[1]].name.c_str());
}

void TimeTableDialog::update()
{
	assert (widgets.find (1234) != widgets.end());
	
	lb = dynamic_cast <ListBoxEx *> (widgets[1234]);
	
	assert (lb != NULL);
		
	for (int j = 0; j < (int)engine.city.size(); ++j)
	{
		for (int i = 0; i < (int)engine.city.size(); ++i)
		{
			char tmp[64] = "-";
			int time = engine.getArrivalTime(i, j);
			if (time >= 0) usprintf(tmp, "%i", time);

			// We could use something like list1.GetItem(i)->SetText(tmp, j), or
			// we could remember the pointer we created when inserting the item
			// and use that to enter the text, but calling SetItem() is just so
			// much easier.
			lb->SetItem(j, i+1, tmp);
		}		
	}

	assert (widgets.find (1003) != widgets.end());	
	Label *lbl = dynamic_cast <Label *> (widgets[1003]);	
	assert (lbl != NULL);
		
	lbl->SetNumber (engine.connectionsLeft);	
}

TimeTableDialog::TimeTableDialog() : XmlDialog()
{
	readFromXml (resources.getDialog ("timetable"));		
	
	myTimer = Timer::Install(500);	
	
	lb = dynamic_cast <ListBoxEx *> (widgets[1234]);
	
	assert (lb != NULL);
	
	lb->InsertColumn("\\", 0, 2, 64);
	for (int i = 0; i < (int)engine.city.size(); ++i)
	{
		// Insert the column at position i, with centre alignment of the
		lb->InsertColumn(engine.city[i].name.c_str(), i+1, 2, 80);
	}
	
	for (int j = 0; j < (int)engine.city.size(); ++j)
	{
		lb->InsertItem(new ListBoxEx::Item());
		lb->SetItem (j, 0, engine.city[j].name.c_str());
		for (int i = 0; i < (int)engine.city.size(); ++i)
		{
			char tmp[64] = "-";
			int time = engine.getArrivalTime(i, j);
			if (time >= 0) usprintf(tmp, "%i", time);

			// We could use something like list1.GetItem(i)->SetText(tmp, j), or
			// we could remember the pointer we created when inserting the item
			// and use that to enter the text, but calling SetItem() is just so
			// much easier.
			lb->SetItem(j, i+1, tmp);
		}
		
	}

	update();
	
}


void CityDialog::HandleEvent(Widget &obj, int msg, int arg1, int arg2) 
{
	Window::HandleEvent (obj, msg, arg1, arg2);
	
}


void StationDialog::MsgTimer (int id)
{
	if (id == myTimer)
	{
		update();
	}
}

void TrainDialog::MsgTimer (int id)
{
	if (id == myTimer)
	{
		update();
	}
}

void TimeTableDialog::MsgTimer (int id)
{
	if (id == myTimer)
	{
		update();
	}
}
