#include "isocanvas.h"
#include "engine.h"
#include "resources.h"
#include "map.h"
#include <sstream>
#include <math.h>
#include <MASkinG.h>
#include <stdlib.h>
#include <iostream>

using namespace MAS;

Engine engine;
Render render;
	
Train::Train() : isDead (false), program(), currentStep(-1), 
	id (-1), dir(N), speed(0), acceleration (0.0005f), max_speed (0.03f),
	last_node_x(-1), last_node_y(-1), 
	hasPassengers (false),
	idCityPassengersSrc (-1),
	passengersDepartureTick (-1),
	steps (1.0), path()
{
}

Station::Station() : isDead (false), id (-1), idCity (-1), name (), x(0), y(0), w(1), h(1)
{
}

City::City() : isDead (false), id (-1), name(), population(0), center_x(0), center_y(0), timeTable()
{
}

Engine::Engine() : temp_map(NULL), nodes(), displayLabels(true), fullRedraw(true), tick(0), money(0), map(), city(), train(), station()
{
}

Engine::~Engine()
{
	if (temp_map) delete[] temp_map;
}

int random (int i)
{
	return rand() % i;
}

void Engine::generateMap()
{
	//~ // add trees
	
	for (int i = 0; i < 250; ++i)
	{
		map.setBuildingTile (random (map.getw()), random(map.geth()), random(2) + 7);
	}
	
	//~ // add cities

	int radius = 10;
	const char *city_names[20] = {
		"Amsterdam", "Rotterdam", "Den Haag", "Maastricht",
		"Wageningen", "Groningen", "Utrecht", "Delft",
		"Leeuwarden", "Arnhem", "Nijmegen", "Tilburg",
		"Eindhoven", "Tilburg", "Zwolle", "Bergen op Zoom",
		"Breda", "Enschede", "Almelo", "Venlo"
	};
	
	for (int i = 0; i < 8; ++i)
	{
		int x;
		int y; 
		bool valid = false;
		while (!valid)
		{
			x = random (map.getw() - 20) + 10;
			y = random (map.geth() - 20) + 10;
			
			valid = true;
			for (vector<City>::iterator j = city.begin(); j != city.end(); ++j)
			{
				int dx = j->center_x - x;
				int dy = j->center_y - y;
				dx *= dx;
				dy *= dy;
				if (dx + dy < radius * radius) 
				{
					valid = false;
					break;
				}
			}			
		}
		
		int idCity = addCity (x, y, city_names[i]);
		city[idCity].population = random (500000) + 50000;
		int p = city[idCity].population;
		while (p > 0)
		{
			int nx = x + random(5) + random(3) + random(3) - 4;
			int ny = y + random(5) + random(3) + random(3) - 4;
			
			if (nx < 0) nx = 0;
			if (nx >= map.getw()) nx = map.getw() -1;
			if (ny < 0) ny = 0;
			if (ny >= map.geth()) ny = map.geth() - 1;

			map.setBuildingTile (nx, ny, random(6));
			p -= 3000;
			
		}
		map.setBuildingTile (x, y, 6); // church in the center
	}
	
}

void Engine::initGame()
{
	map.init (64, 64);
	
	station.clear();
	train.clear();
	city.clear();

	generateMap();
	
	connectionsLeft = city.size() * (city.size() - 1);
	
	money = 2000;
	tick = 0;
	
	//~ int q = 32;
	//~ for (int i = 1; i <= q; ++i)
	//~ {
		
		//~ map.setTrack (i, 1, true);
		//~ map.setTrack (i, q, true);
		//~ map.setTrack (q, i, true);
		//~ map.setTrack (1, i, true);
		//~ map.setTrack (q/2, i, true);
		//~ map.setTrack (i, q/2, true);
	//~ }
		
	//~ addStation (0, q/3);
	//~ addStation (q+1, q*2/3);
	//~ addStation (q/4, q+1);
	
	//~ addTrain (0);
	//~ addTrain (1);	
	//~ addTrain (2);	
}

void Engine::init()
{
	// create map	
	map.init (64, 64);
	temp_map = new AstarTile[map.getw() * map.geth()];

	initGame();
}

// returns -2: not enough money
// returns -1: no city nearby
int Engine::addStation (int mx, int my)
{	
	Station s;
	
	// find nearest city	
	s.idCity = -1;
	int min_d = 2 * engine.map.getw() + 1;
	for (vector<City>::iterator i = engine.city.begin();
		i != engine.city.end(); ++i)
	{
		int d = abs (mx - i->center_x) + abs (my - i->center_y);
		if (d < min_d)
		{
			s.idCity = i->id; 
			min_d = d;
		}
	}
	
	if (s.idCity == -1)
	{
		return -1;
	}

	if (money < 500)
	{
		return -2;
	}
	money -= 500;
	
	s.id = station.size();
	// TODO: search for a free id;
		
	s.x = mx;
	s.y = my;
	
	map.tile[mx + map.getw() * my].idStation = s.id;
	map.setTrack (mx, my, true);
	
	{ // decide on a name
		int dx = mx - city[s.idCity].center_x;
		int dy = my - city[s.idCity].center_y;
		
		s.name = city[s.idCity].name;
		
		if (abs(dx) + abs (dy) < 10)
		{
			s.name += " centraal";
		}
		else if (abs(dx) < abs(dy))
		{
			if (dy < 0) s.name += " noord";
				else s.name += " zuid";
		}
		else
		{
			if (dy > 0) s.name += " oost";
				else s.name += " west";
		}
	}
	
	station.push_back (s);
	
	return s.id;
}

void Engine::onArrive (int idTrain, int idStation)
{
	// do bookkeeping
	
	if (train[idTrain].hasPassengers)
	{
		int idSrcCity = train[idTrain].idCityPassengersSrc;
		int idDestCity = station[idStation].idCity;
		int dTime = tick - train[idTrain].passengersDepartureTick;
		
		assert (idSrcCity >= 0);
		assert (idDestCity >= 0);
		assert (dTime >= 0);
		
		int dSpace = abs (city[idSrcCity].center_x - city[idDestCity].center_x)
			+ abs (city[idSrcCity].center_y - city[idDestCity].center_y);
		
		int dMoney = dSpace * 1000 / dTime;
		engine.money += dMoney;
		
		ostringstream msg1;
		
		msg1 << "Hooray! Train " << idTrain << 
			" arrived to " << station[idStation].name;
		msg1 << " with passengers from " << city[idSrcCity].name << 
			", after " << dTime << " ticks. ";
		msg1 << " You earned "
			<< " $ " << dMoney << "!";
			
		lblDebug->SetText (msg1.str().c_str());
		setArrivalTime (idSrcCity, idDestCity, dTime);
	}
	
	// load up passengers
	train[idTrain].hasPassengers = true;
	train[idTrain].idCityPassengersSrc = station[idStation].idCity;
	train[idTrain].passengersDepartureTick = tick;
}

int Engine::addCity(int mx, int my, string name)
{
	City c;

	c.center_x = mx;
	c.center_y = my;
	c.name = name;

	c.id = city.size();
	city.push_back (c);
	return c.id;
}

// -1, couldn't add, not enough stations
// -2, not enough money
int Engine::addTrain(int idStation)
{
	if (money < 100)
	{
		return -2;
	}
	money -= 100;
	
	if (engine.station.size() < 2)
	{
		return -1;
	}		
	
	Train t;
	t.dir = N;
	
	int mx = station[idStation].x;
	int my = station[idStation].y;
	
	t.last_node_x = mx;
	t.last_node_y = my;
	
	t.program.push_back (idStation);
	
	// simple program generator:
	int ii = idStation;
	while (ii == idStation)
	{
		ii = random (engine.station.size());
	}
	t.program.push_back (ii);
	t.currentStep = 1;
	
	// TODO: search for a free id;
	
	t.id = train.size();
	
	map.tile[mx + map.getw() * my].idTrain = t.id;
	
	train.push_back (t);
	
	return t.id;
}

void Train::doMove()
{
	speed += acceleration;
	if (speed > max_speed) 
	{
		speed = max_speed;
	}
	
	if (steps + speed >= 1.0)
	{
		steps = 0.0;
		
		// update map status		
		engine.map.tile[last_node_x + engine.map.getw() * last_node_y].idTrain = -1;
		
		last_node_x = getNextNodeX();
		last_node_y = getNextNodeY();
		
		// update map status
		engine.map.tile[last_node_x + engine.map.getw() * last_node_y].idTrain = id;
		
		// now decide to change direction...		
		
		if (engine.isGoal (id, last_node_x, last_node_y))
		{
			// yes! we arrived at our goal
			engine.onArrive (id, getCurrentGoal());
			
			// advance program
			currentStep++;
			if (currentStep >= (int)program.size()) currentStep = 0;
			path.clear(); // path should be empty, but clear it anyway
		}
	
		if (path.size() == 0)
		{
			engine.createPath (id);	
			
			if (path.size() == 0) // still can't reach goal
			{
				// change program randomly:
				int ii = random (engine.station.size());
				while (ii == program[0] || ii == program[1])
				{
					ii = random (engine.station.size());
				}
				assert (currentStep == 0 || currentStep == 1);
				program[currentStep] = ii;
				assert (ii < (int)engine.station.size());
				assert (ii >= 0);
				assert (engine.station[ii].isDead == false);
			}
		}

		bool valid = false;
		// try to follow path
		if (path.size() > 0)
		{
			int d = path.front();
			path.pop_front();
			
			assert (d >= 0);
			assert (d < 4);
			
			if (engine.canGo (last_node_x, last_node_y, d))
			{
				if (dir != d)
				{
					dir = d;
					speed /= 2;
				}
				valid = true;
			}
			else
			{
				// path turns out to be invalidated
				path.clear();			
				valid = false;
			}
		}
		
		if (!valid)		
		{
			// ga ijsberen...
			int d = dir;
			d++; d %= 4;
			int i = 0;
			
			while (!engine.canGo (last_node_x, last_node_y, d) && i < 4)
			{
				d++; // ok we move
				d %= 4;
				i++;
			}
			if (dir != d)
			{
				dir = d;
				speed /= 2;
			}
		}
		if (!engine.canGo (last_node_x, last_node_y, dir))
		{
			speed = 0;
		}
	}

	steps += speed;	
}

void Engine::update()
{
	tick++;
	
	vector<Train>::iterator i;
	for (i = train.begin(); i != train.end(); ++i)
	{
		i->doMove();
		
		if (!fullRedraw)
		{
			int x = render.canvasFromMapX (i->getLx(), i->getLy());
			int y = render.canvasFromMapY (i->getLx(), i->getLy());
			isoCanvas->markCanvasDirty (x + 16, y, 32, 32);
		}
	}
	
	if (fullRedraw)
	{
		// full redraw, use if having problems with dirty rectangles.
		// but will cause high CPU usage
		isoCanvas->Redraw();
	}
}

void Render::init()
{
	// resources already inited at this point.
	// obtain arrays of tiles
	
	
	tracks[ 0] = resources.getRle ("grass");
	tracks[ 1] = resources.getRle ("via-media1");
	tracks[ 2] = resources.getRle ("via-media3");
	tracks[ 3] = resources.getRle ("cruce2");
	tracks[ 4] = resources.getRle ("via-media2");
	tracks[ 5] = resources.getRle ("via1");
	tracks[ 6] = resources.getRle ("cruce5");
	tracks[ 7] = resources.getRle ("cruce-t2");
	tracks[ 8] = resources.getRle ("via-media4");
	tracks[ 9] = resources.getRle ("cruce4");
	tracks[10] = resources.getRle ("via2");
	tracks[11] = resources.getRle ("cruce-t4");
	tracks[12] = resources.getRle ("cruce3");
	tracks[13] = resources.getRle ("cruce-t3");
	tracks[14] = resources.getRle ("cruce-t1");
	tracks[15] = resources.getRle ("cruce1");
	tracks[16] = resources.getRle ("station1");
	tracks[17] = resources.getRle ("station2");
	
	const char *building_names[9] = {
		"casas1", "casas2", "casas3", "casas4", 
		"casas5", "casas6", "church", "tree1", "tree2"
	};
	
	for (int i = 0; i < 9; ++i)
	{
		buildings[i] = resources.getRle (building_names[i]);
	}
	
	wagon[0] = resources.getRle ("trein1");
	wagon[1] = resources.getRle ("trein2");
}

int Render::canvasFromMapX (int mx, int my) const
{
	return getw() / 2 + mx * 32 + my * -32;
}

int Render::canvasFromMapY (int mx, int my) const
{
	return mx * 16 + my * 16;
}

int Render::canvasFromMapX (float mx, float my) const
{
	return (int)(getw() / 2 + mx * 32.0 + my * -32.0);
}

int Render::canvasFromMapY (float mx, float my) const
{
	return (int)(mx * 16.0 + my * 16.0);
}

int Render::mapFromCanvasX (int x, int y) const
{
	return ((x - (getw() / 2) - 32) / 2 + y) / 32;
}

int Render::mapFromCanvasY (int x, int y) const
{
	return  (y - (x - (getw() / 2) - 32) / 2) / 32;
}

void Render::draw(BITMAP *canvas, int xofst, int yofst)
{
	int mx, my;
	for (mx = 0; mx < engine.map.getw(); ++mx)
	{
		for (my = 0; my < engine.map.geth(); ++my)
		{
			draw_rle_sprite (canvas, tracks[engine.map.getTrackTile(mx, my)], canvasFromMapX(mx, my) - xofst, canvasFromMapY(mx, my) - yofst);
		}
	}
	for (mx = 0; mx < engine.map.getw(); ++mx)
	{
		for (my = 0; my < engine.map.geth(); ++my)
		{
			int building = engine.map.getBuildingTile(mx, my);
			if (building != -1)
			{
				draw_rle_sprite (canvas, buildings[building], 
					canvasFromMapX(mx, my) - xofst, 
					canvasFromMapY(mx, my) - yofst + 32 - buildings[building]->h);
			}
		}
	}
	
	vector <Train>::iterator i;
	for (i = engine.train.begin(); i != engine.train.end(); ++i)
	{
		RLE_SPRITE *s = wagon[0];
		if (i->dir == E || i->dir == W) s = wagon[1];
		draw_rle_sprite (canvas, s, 
			canvasFromMapX (i->getLx(), i->getLy()) - xofst - s->w / 2 + 32,
			canvasFromMapY (i->getLx(), i->getLy()) - yofst - s->h / 2 + 16);
	}
	
	if (engine.displayLabels)
	{
		// city names
		for (vector <City>::iterator c = engine.city.begin();
			c != engine.city.end(); ++c)
		{
			textout_centre_ex(canvas, font, c->name.c_str(), 
				canvasFromMapX (c->center_x, c->center_y) - xofst + 32,
				canvasFromMapY (c->center_x, c->center_y) - yofst + 16,
				makecol (255, 255, 255), makecol (64, 64, 64));
		}
	
		// station names
		for (vector <Station>::iterator c = engine.station.begin();
			c != engine.station.end(); ++c)
		{
			textout_centre_ex(canvas, font, c->name.c_str(), 
				canvasFromMapX (c->x, c->y) - xofst + 32,
				canvasFromMapY (c->x, c->y) - yofst + 16,
				makecol (0, 255, 255), makecol (64, 64, 64));
		}
	}
	
	// Debug: draw temp_map of pathfinder
	
	//~ for (mx = 0; mx < engine.map.getw(); ++mx)
	//~ {
		//~ for (my = 0; my < engine.map.geth(); ++my)
		//~ {
			//~ int cell = mx + my * engine.map.getw();
			
			//~ int x = canvasFromMapX(mx, my) - xofst + 32;
			//~ int y = canvasFromMapY(mx, my) - yofst + 16;
			
			//~ if (engine.temp_map[cell].state == OPEN ||
				//~ engine.temp_map[cell].state == CLOSED)
			//~ {
				//~ circlefill (canvas, x, y, 5, makecol (255, 255, 0));
				//~ int px = engine.temp_map[cell].parent_x;
				//~ int py = engine.temp_map[cell].parent_y;				
				//~ int x2 = canvasFromMapX(px, py) - xofst + 32;
				//~ int y2 = canvasFromMapY(px, py) - yofst + 16;
				//~ line (canvas, x, y, x2, y2, makecol (255, 255, 0));
			//~ }
		//~ }
	//~ }

	// Debug: draw pathfinder paths
	
	//~ for (i = engine.train.begin(); i != engine.train.end(); ++i)
	//~ {
		//~ int x = canvasFromMapX(i->getNextNodeX(), i->getNextNodeY()) - xofst + 34;
		//~ int y = canvasFromMapY(i->getNextNodeX(), i->getNextNodeY()) - yofst + 18;
		
		//~ list<int>::iterator j;
		//~ for (j = i->path.begin(); j != i->path.end(); ++j)
		//~ {
			//~ int x2, y2;
			//~ x2 = x + 32 * Train::getDx(*j) - 32 * Train::getDy (*j);
			//~ y2 = y + 16 * Train::getDx(*j) + 16 * Train::getDy (*j);
			
			//~ line (canvas, x, y, x2, y2, makecol (255, 0, 0));
			//~ x = x2;
			//~ y = y2;
		//~ }
	//~ }	
}

int Render::getw() const { return engine.map.getw() * 64; }
int Render::geth() const { return engine.map.geth() * 32; }

// part of astar pathfinder
float Engine::getHeuristics (int sx, int sy, int dx, int dy)
{
	// manhattan distance
	int d = abs (sx - dx) + abs (sy - dy);
	return (float)(d) * 1.01;
}

// part of astar pathfinder
void Engine::examineCell (int idTrain, int x, int y, int d)
{
	if (!canGo (x, y, d)) return;
	
	int nx = x + Train::getDx (d);
	int ny = y + Train::getDy (d);
	if (x < 0 || y < 0 || x >= map.getw() || y >= map.geth()) return;
	
	int parentCell = x + map.getw() * y;
	int examineCell = nx + map.getw() * ny;
	
	float newg = temp_map[parentCell].g + 1.0;

	if ((temp_map[examineCell].state == OPEN || 
		temp_map[examineCell].state == CLOSED) &&
		temp_map[examineCell].g <= newg)
	{
		// skip
	}
	else
	{	
		Train *t = &train[idTrain];	
		int idGoalStation = t->getCurrentGoal();
		int goal_x = station[idGoalStation].x;
		int goal_y = station[idGoalStation].y;
		
		openCell (nx, ny, 
			x, y, // parent
			newg,
			getHeuristics (nx, ny, goal_x, goal_y)
		);
	}
}

// astar pathfinder
void Engine::createPath (int idTrain)
{
	Train *t = &train[idTrain];	
	int idGoalStation = t->getCurrentGoal();
	
	assert (idGoalStation >= 0);
	assert (station[idGoalStation].isDead == false);
	
	int goal_x = station[idGoalStation].x;
	int goal_y = station[idGoalStation].y;
	
	// clear temp map.
	//memset (temp_map, -1, sizeof (temp_map));
	for (int i = 0; i < map.getw() * map.geth(); ++i)
	{
		temp_map[i].state = UNDEFINED;
		temp_map[i].parent_x = -1;
		temp_map[i].parent_y = -1;		
	}
	
	nodes.clear();

	// set very first... our start cell
    openCell (t->last_node_x, t->last_node_y,
		-1, -1, // parent
		0.0, // g
		getHeuristics (t->last_node_x, t->last_node_y, goal_x, goal_y)
	); 

    bool valid         = true;
    bool succes        = false;
	int last_x = -1;
	int last_y = -1;
    
    // WHILE VALID TO RUN THIS LOOP
    while (valid)	
	{
		int curx, cury;
		getBestOpenCell (curx, cury);
		
		if (curx == -1)
		{
			valid = false;
			succes = false;
			break;
		}
		
		if (isGoal (idTrain, curx, cury))
		{
			valid = false;
			succes = true;
			last_x = curx;
			last_y = cury;
			break;
		}
		
		examineCell (idTrain, curx, cury, N);
		examineCell (idTrain, curx, cury, E);
		examineCell (idTrain, curx, cury, S);
		examineCell (idTrain, curx, cury, W);
		
	}
	
	//~ // now store path
	if (succes)
	{
		while (1)
		{
			int last_cell = last_x + map.getw() * last_y;
			int px = temp_map[last_cell].parent_x;
			int py = temp_map[last_cell].parent_y;
			
			if (px < 0 || py < 0)
			{
				// done
				break;
			}

			int dx = last_x - px;
			int dy = last_y - py;
			
			assert (abs(dx) + abs (dy) == 1);
			// try to figure out dir
			int dir = 0;
			
			while (!(dx == Train::getDx (dir) && dy == Train::getDy (dir))
					&& dir < 4) { dir++; }
					
			t->path.push_front (dir);
					
			last_x = px;
			last_y = py;
		}		
		return; // success
    }
	
	return; // failure
}

// currently equal to map.hasTrack, but this will change
// when traffic lights arrive
bool Engine::canGo (int mx, int my, int d)
{
	return map.hasTrack (mx + Train::getDx(d), my + Train::getDy(d));
}

// have we arrived at our goal trainstation?
bool Engine::isGoal (int idTrain, int mx, int my)
{
	int id = map.tile[mx + map.getw() * my].idStation;	
	if (id >= 0 && id == train[idTrain].getCurrentGoal())
	{
		return true;
	}
	return false;
}

// comparison function for astar pathfinder
bool nodeByF(const AstarNode &a, const AstarNode &b)
{
	return a.f > b.f;
}

// part of astar pathfinder
void Engine::openCell (int mx, int my, int px, int py, float newg, float newh)
{
	int cell = mx + map.getw() * my;
	
	// add to list if it wasn't in there
	if (temp_map[cell].state != OPEN)
	{
		AstarNode n;
		n.f = newg + newh;
		n.mx = mx;
		n.my = my;
		nodes.push_back (n);
		push_heap (nodes.begin(), nodes.end(), nodeByF);
	}
	
	temp_map[cell].h = newh;
    temp_map[cell].g = newg;
    temp_map[cell].parent_x = px;
	temp_map[cell].parent_y = py;
    temp_map[cell].state = OPEN;
}

// part of astar pathfinder
// returns  -1, -1 if none found
void Engine::getBestOpenCell (int &x, int &y)
{
	if (nodes.empty())
	{
		x = -1;
		y = -1;
	}
	else
	{
		x = nodes.front().mx;
		y = nodes.front().my;
		pop_heap (nodes.begin(), nodes.end(), nodeByF);		
		nodes.pop_back();
		temp_map[x + map.getw() * y].state = CLOSED;
	}
}


void Engine::setArrivalTime (int idSrcCity, int idDestCity, int dTime)
{
	if (idSrcCity == idDestCity) return;
	
	// look in dest city
	
	if (city[idDestCity].timeTable.find(idSrcCity) == 
		city[idDestCity].timeTable.end())
	{
		connectionsLeft--;
		city[idDestCity].timeTable.insert(pair<int, int>(idSrcCity, dTime));
	}
	else
	{
		if (dTime < city[idDestCity].timeTable[idSrcCity])
		{
			city[idDestCity].timeTable[idSrcCity] = dTime;
		}
	}
}

// returns -1 if no arrival time
int Engine::getArrivalTime (int idSrcCity, int idDestCity)
{
	if (city[idDestCity].timeTable.find(idSrcCity) == 
		city[idDestCity].timeTable.end())
	{
		return -1;
	}
	else
	{
		return city[idDestCity].timeTable[idSrcCity];
	}
}
