#include "isoCanvas.h"
#include <sstream>
#include "mainDlg.h"
#include "dialogs.h"

using namespace MAS;

IsoCanvas *isoCanvas;
Label *lblDebug;

void IsoCanvas::DrawClientArea (Bitmap &canvas, int xofst, int yofst) {
	canvas.Clear(Color::black);
	
	render.draw (canvas, xofst, yofst);
	
	if (cursor_x >= 0 && cursor_y >= 0 &&
		cursor_x < engine.map.getw() && cursor_y < engine.map.geth())
	{
		int x = render.canvasFromMapX (cursor_x, cursor_y) - xofst;
		int y = render.canvasFromMapY (cursor_x, cursor_y) - yofst;
		
		Point p1(x + 31, y);
		Point p2(x + 63, y + 15);
		Point p3(x + 31, y + 31);
		Point p4(x, y + 15);
		
		canvas.Line (p1, p2, makecol (255, 255, 0));
		canvas.Line (p2, p3, makecol (255, 255, 0));
		canvas.Line (p3, p4, makecol (255, 255, 0));
		canvas.Line (p4, p1, makecol (255, 255, 0));
		
		cursorRect = Rect (x, y, 66, 34); // store for marking dirty later
	}
	// display dirty rectangles:
	//~ Rect r = canvas.GetClip();
	//~ canvas.Rectangle (r.x(), r.y(), r.x2()-1, r.y2()-1, makecol (255, 0, 0));			

}
	
IsoCanvas::IsoCanvas() : ScrollBox(), cursor_x(-1), cursor_y(-1) {
	
	SetScrollSize (render.getw() + 64, render.geth());
	Center();

	// install a timer that will tick at 100 ms intervals;
	scrollTimer = Timer::Install(100);	
}
		
IsoCanvas::~IsoCanvas() 
{	
	Timer::Kill(scrollTimer);
	scrollTimer = -1;

}

void IsoCanvas::MsgMousemove(const Point &d)
{
	//~ ostringstream msg;
	Point p = GetMousePos();	
	Point q = Point (
		p.x() - GetClientLeft() + GetXOffset(), 
		p.y() - GetClientTop() + GetYOffset()
	);
	//~ msg << "p: (" << q.x() << ", " << q.y() << ") mx: (" << render.mapFromCanvasX (q.x(), q.y()) <<
	 //~ ", " << render.mapFromCanvasY (q.x(), q.y()) << ")";
	//~ lblDebug->SetText (msg.str().c_str());
		
	cursor_x = render.mapFromCanvasX (q.x(), q.y());
	cursor_y = render.mapFromCanvasY (q.x(), q.y());
	
	if (engine.fullRedraw)
	{
		Redraw();
	}
	else
	{
		Redraw (cursorRect); // remove old rect
		Redraw (Rect (p.x() - 64, p.y() - 32, 128, 64));
	}
}

// called one time per click
void IsoCanvas::MsgLPress()
{
    Point p = GetMousePos();
	
	MainDlg *mainDlg = dynamic_cast<MainDlg*>(GetParent());
	assert (mainDlg);
	
	if (cursor_x >= 0 && cursor_y >= 0 &&
		cursor_x < engine.map.getw() && cursor_y < engine.map.geth())
	{
		
		switch (mainDlg->getEditState())
		{
			case 0: // just pointer
			{
				int cell = cursor_x + engine.map.getw() * cursor_y;
				
				if (engine.map.tile[cell].idStation >= 0)
				{
					// popup station dialog
					StationDialog *td = new StationDialog();
					td->setStation(engine.map.tile[cell].idStation);
					GetParent()->Add (*td);
					// TODO: memleak, xd not deleted anywhere
				}
				else if (engine.map.tile[cell].idTrain >= 0)
				{
					TrainDialog *td = new TrainDialog();
					td->setTrain(engine.map.tile[cell].idTrain);
					GetParent()->Add (*td);
					// TODO: memleak, xd not deleted anywhere
					// popup train dialog
				}
				// TODO: popup city dialog
				break;
			}
			case 1:
			{
				int result = engine.map.setTrack (cursor_x, cursor_y, true);
				if (result == -2)
				{
					MessageBox msg("No Money", "You don't have money for that", NULL, NULL, "OK");
					msg.Popup (this);
				}
				else
				{
					Redraw();
				}
				break;
			}
			case 2:
			{
				int result = engine.map.setTrack (cursor_x, cursor_y, false);
				if (result == -2)
				{
					MessageBox msg("No Money", "You don't have money for that", NULL, NULL, "OK");
					msg.Popup (this);
				}
				else
				{
					Redraw();
				}				
				break;
			}
			case 3:
			{
				// add station
				int result = engine.addStation (cursor_x, cursor_y);
				if (result == -2)
				{
					MessageBox msg("No Money", "You don't have money for that", NULL, NULL, "OK");
					msg.Popup (this);
				}
				else if (result == -1)
				{
					MessageBox msg("Error", "Not near to a city", NULL, NULL, "OK");
					msg.Popup (this);
				}
				else
				{
					Redraw();
					
					// popup station dialog
				}
				break;
			}
			case 4:
				// traffic lights
				break;
			default:
				assert (false);
		}
    }
}

void IsoCanvas::MsgTimer(int id) {
	Dialog::MsgTimer(id);

	// MsgTimer() is called whenever a timer ticks. There could be other
	// timers installed (actually there is at least one - the system timer)
	// so we need to check if this call was actually made by our stopwatch
	// timer.
	if (id == scrollTimer) {
		Point p = GetMousePos();
	
		const int scrollStep = 32;
		if (p.x() <= 0)
		{
			SetPos (GetXOffset() - scrollStep,  GetYOffset());
		}
		if (p.x() >= w()-1)
		{
			SetPos (GetXOffset() + scrollStep,  GetYOffset());
		}
		if (p.y() <= 0)
		{
			SetPos (GetXOffset(),  GetYOffset() - scrollStep);
		}
		if (p.y() >= h()-1)
		{
			SetPos (GetXOffset(),  GetYOffset() + scrollStep);
		}		
	}
}

void IsoCanvas::markCanvasDirty (int x1, int y1, int w, int h)
{
	Redraw (Rect (x1 - GetXOffset(), y1 - GetYOffset(), w, h));
}
