#include "mainDlg.h"
#include <assert.h>
#include "XmlDialog.h"
#include <sstream>

bool MainDlg::MsgClose() 
{
    Dialog::MsgClose();
    MessageBox msg("quit", "Do you really want to quit?",
                        NULL, NULL, "&Yes", "&No", NULL);
    if (msg.Popup(this) == 1) 
    {
        return true;
    }
    return false;
}

int MainDlg::menu_about()
{
    Window dlg;                    

    Button bOK;   
    PanelRaised theBox;                    // a raised panel
    TextArea ta;
	
    theBox.Shape (0,0,560,360);
    bOK.Setup (240,310,80,24, KEY_ENTER, D_EXIT, "&OK");
    ta.LoadLinesFromFile ("about.txt");
    ta.Shape (10,10,540,300);
    
    // resize the dialog to the same size as the panel
    dlg.Resize(560, 360);
     
    // add the objects to the dialog
    dlg.Add(theBox);
    dlg.Add(bOK);
    dlg.Add(ta);
     
    // centre the dialog on the screen
    dlg.Centre();
     
    // execute the dialog
    dlg.Popup(this);
	return D_O_K;
}

int MainDlg::menu_help()
{
    Window dlg;                     

    Button bOK;   
    PanelRaised theBox;                    // a raised panel
    TextArea ta;
    
    bOK.Setup (240,310,80,24, KEY_ENTER, D_EXIT, "&OK");
    theBox.Shape (0,0,560,360);
    ta.LoadLinesFromFile ("help.txt");
    ta.Shape (10,10,540,300);
    
    // resize the dialog to the same size as the panel
    dlg.Resize(560, 360);
     
    // add the objects to the dialog
    dlg.Add(theBox);
    dlg.Add(bOK);
    dlg.Add(ta);
    
    // centre the dialog on the screen
    dlg.Centre();
     
    // execute the dialog
    dlg.Popup(this);
	return D_O_K;
}


int MainDlg::menu_new (void)
{
	MessageBox msg("New", "Do you really want to start a new game?",
                        NULL, NULL, "&Yes", "&No", NULL);
    if (msg.Popup(this) == 1) 
    {
        engine.init();
    }
		
    return D_O_K;
}

int MainDlg::menu_load (void)
{
	MessageBox msg("Not Implemented", "Due to time constraints,", 
		"this feature is not yet implemented. Sorry.", NULL, "OK");
	msg.Popup(this);
	
    return D_O_K;
}

int MainDlg::menu_full()
{
	engine.fullRedraw = !engine.fullRedraw;

	if (engine.fullRedraw) menuFile.Check (5);
		else menuFile.Uncheck(5);

	return D_O_K;
}
	

int MainDlg::menu_save (void)
{
    MessageBox msg("Not Implemented", "Due to time constraints,", 
		"this feature is not yet implemented. Sorry.", NULL, "OK");
	msg.Popup(this);
	
	return D_O_K;
}

int MainDlg::menu_timetable ()
{
	TimeTableDialog *td = new TimeTableDialog();
	Add (*td);
	// TODO : memory leak. td is never deleted
	return D_O_K;
}

MainDlg::MainDlg() : Dialog()
{   
    Add(desktop);

	ic.Shape (0, 0, 640, 464);
	Add (ic);
	isoCanvas = &ic;
	
    MakeMenu();
	
	ctrl_l.Setup (KEY_L, KB_CTRL_FLAG);
    Add(ctrl_l);
    ctrl_q.Setup (KEY_Q, KB_CTRL_FLAG);
    Add(ctrl_q);
    ctrl_n.Setup (KEY_N, KB_CTRL_FLAG);
    Add(ctrl_n);
    ctrl_s.Setup (KEY_S, KB_CTRL_FLAG);
    Add(ctrl_s);
    f1.Setup (KEY_F1);
    Add (f1);
	f2.Setup (KEY_F2);
	Add (f2);

	const char *btn_labels[] = {
		"P", "+R", "-R", "S", "TL"
	};
	
	const int btn_key[] = {
		KEY_1, KEY_2, KEY_3, KEY_4, KEY_5
	};
	
	const char *btn_tooltip[] = {
		"Pointer", "Build more railroads", "Remove railroads", "Build new stations", "Traffic Lights"
	};
	
	for (int i = 0; i < 5; ++i)
	{
		toolbar[i].Setup (320 + 32 * i, 0, 32, 16, btn_key[i], D_TOGGLE, btn_labels[i]);
		toolbar[i].SetTooltipText (btn_tooltip[i]);
		Add (toolbar[i]);
	}
	toolbar[0].Select();
	toolbar[4].Disable();
	edit_state = 0;
	
	lbl.Shape (0, 20, 320, 32);
	lbl.ClearFlag(D_AUTOSIZE);	
	lbl.SetWordWrap (true);
	lbl.SetTextMode (makecol (192, 192, 192));
	lblDebug = &lbl;	

	pnlStatus.Shape (0, 464, 640, 16);
	Add (pnlStatus);
	
	lblMoney.Shape(80, 466, 160, 12);	
	lblMoney.ClearFlag(D_AUTOSIZE);	
	lblMoney.SetVAlignment(2);
	lblMoney.SetTextMode (makecol (192, 192, 192));
	lblMoney.SetText("");
	Add (lblMoney);
	
	lblClock.Shape (240, 466, 160, 12);
	lblClock.ClearFlag(D_AUTOSIZE);	
	lblClock.SetVAlignment(2);
	lblClock.SetTextMode (makecol (192, 192, 192));
	lblClock.SetText("");
	Add (lblClock);
	
	Add (lbl);
    Add(menuMain);    

	// install a timer that will tick at 100 ms intervals;
	timerStatusPanel = Timer::Install(100);	
}

MainDlg::~MainDlg() 
{
	Timer::Kill(timerStatusPanel);
}

void MainDlg::MakeMenu() 
{
    menuFile.Add("&New\tCtrl-N", FILE_NEW);
    menuFile.Add("&Load Map\tCtrl-L", FILE_LOAD);
    menuFile.Add("&Save Map\tCtrl-S", FILE_SAVE);
    menuFile.Add();
    menuFile.Add("&Time Table\tF2", FILE_TIME);
    menuFile.Add("Toggle &Full Redraw", FILE_FULL);
	
	if (engine.fullRedraw) menuFile.Check (5);

	menuFile.Add();
    menuFile.Add("&Quit\tCtrl-Q", FILE_QUIT);
    
    menuHelp.Add("&Contents\tF1", HELP_HELP);
    menuHelp.Add("&About", HELP_ABOUT);
    
    menuMain.Add("&File", menuFile);
    menuMain.Add("&Help", menuHelp);
}

void MainDlg::SwitchState (int new_state)
{
	assert (new_state >= 0);
	assert (new_state < 5);
	toolbar[edit_state].Deselect();
	toolbar[new_state].Select();
	edit_state = new_state;
}

void MainDlg::HandleEvent(Widget &obj, int msg, int arg1, int arg2)
{
#ifdef ALLEGRO_LINUX
    if (msg == MSG_IDLE)
    {
        struct timeval tv = {0, 1};
        select (0, NULL, NULL, NULL, &tv);        
    }
#endif    
    if (obj == ctrl_q && msg == MSG_ACTIVATE)
    {
        Close();
    }    
    if (obj == ctrl_n && msg == MSG_ACTIVATE)
    {
        menu_new();
    }
    if (obj == ctrl_s && msg == MSG_ACTIVATE)
    {
        menu_save();
    }
    if (obj == ctrl_l && msg == MSG_ACTIVATE)
    {
        menu_load();
    }
    if (obj == f1 && msg == MSG_ACTIVATE)
    {
        menu_help();
    }
    if (obj == f2 && msg == MSG_ACTIVATE)
    {
        menu_timetable();
    }
	if (obj == toolbar[0] && msg == MSG_ACTIVATE)
	{
		SwitchState (0);
	}
	if (obj == toolbar[1] && msg == MSG_ACTIVATE)
	{
		SwitchState (1);
	}
	if (obj == toolbar[2] && msg == MSG_ACTIVATE)
	{
		SwitchState (2);
	}
	if (obj == toolbar[3] && msg == MSG_ACTIVATE)
	{
		SwitchState (3);
	}
	if (obj == toolbar[4] && msg == MSG_ACTIVATE)
	{
		SwitchState (4);
	}
    if (obj == menuMain) 
    {
       switch (msg) 
       {
          case FILE_NEW:
              menu_new();
              break;
          case FILE_LOAD:
              menu_load();
              break; 
		  case FILE_FULL:
			  menu_full();
		      break;
          case FILE_SAVE:
              menu_save();
              break;
          case FILE_QUIT:
              Close();
              break; 
		  case FILE_TIME:
			  menu_timetable();
			  break;
          case HELP_HELP:
              menu_help();
              break;
          case HELP_ABOUT:
              menu_about();
              break;
          default:
              
              break;
       };
    }
	
	XmlDialog *xobj;
	if ((xobj = dynamic_cast<XmlDialog*>(&obj)) != NULL && msg == MSG_CLOSE)
	{
		Remove (obj);
		//delete xobj;
		// TODO: delete obj
	}
}

void MainDlg::MsgTick()
{
	Dialog::MsgTick();
	engine.update();
}

void MainDlg::MsgTimer(int id)
{
	Dialog::MsgTimer(id);
	
	if (id == timerStatusPanel)
	{
		ostringstream msg, msg2;
		
		msg << "$ " << engine.money;
		lblMoney.SetText (msg.str().c_str());

		int min = engine.getMin();
		int day = min / 60 / 24;
		int hour = (min / 60) % 24;
		min %= 60;
		
		msg2 << "Day " << day << ", " << hour << ":" << min;
		lblClock.SetText (msg2.str().c_str());
	}
}
