#include "map.h"
#include "engine.h"

Map::Map() : mw(0), mh(0), tile(NULL)
{
}

Map::~Map()
{
	if (tile)
		delete[] tile;
}

void Map::init(int w, int h)
{
	if (tile)
		delete[] tile;
	
	mw = w;
	mh = h;
	tile = new Tile[mw * mh];
}

bool Map::hasTrack (int x, int y) const
{
	if (x < 0 || y < 0 || x >= mw || y >= mh)
	{
		return false;
	}
	return tile[x + mw * y].hasTrack;
}

void Map::calcTrackTile(int x, int y)
{
	if (x < 0 || y < 0 || x >= mw || y >= mh)
	{
		return;
	}
	
	int result = 
		(hasTrack (x, y - 1) ? 1 : 0) +
		(hasTrack (x + 1, y) ? 2 : 0) +
		(hasTrack (x, y + 1) ? 4 : 0) +
		(hasTrack (x - 1, y) ? 8 : 0);
	if (hasTrack (x, y))
	{
		// special case: surrounded by nothing, display something.
		if (result == 0) result = 1;
			
		// do we have a station here?
		if (tile[x + mw * y].idStation >= 0)
		{
			if (result & 5) result = 16; else result = 17;
		}
	}
	else
	{
		result = 0;
	}
	tile[x + mw * y].track_tile = result;
}

// returns negative nr on failure
// -1: out of range
// -2: not enough money
// -3: building present
// -4: station present
// 0: ok
int Map::setTrack (int x, int y, bool value)
{
	if (x < 0 || y < 0 || x >= mw || y >= mh)
	{
		return -1;
	}
	
	int oldValue = tile[x + mw * y].hasTrack;
	if (value == oldValue) return 0; // no change

	// comment: we allow building over buildings for now...
	//~ if (tile[x + mw * y].building_tile != -1)
	//~ {
		//~ return -3; // cannot build on top of building
	//~ }

	if (tile[x + mw * y].idStation != -1 && value == false)
	{
		return -4; // cannot remove from station, building has no effect
	}
	
	if (value)
	{
		if (engine.money < 10)
		{
			return -2;
		}
		else
		{
			engine.money -= 10;
		}
	}
	else
	{
		engine.money += 5; // reclaim some of the build costs
	}

	tile[x + mw * y].building_tile = -1; // remove present building tile
	
	tile[x + mw * y].hasTrack = value;
	calcTrackTile (x, y);
	calcTrackTile (x + 1, y);
	calcTrackTile (x, y + 1);
	calcTrackTile (x - 1, y);
	calcTrackTile (x, y - 1);	
	
	return 0;
}

Tile::Tile() : 
	track_tile(0), 	semaphore_tile(0), 
	building_tile(-1), hasTrack(false), semaphore_state (0), 
	idStation(-1), idTrain (-1)
{
}
