#include "resources.h"
#include <assert.h>
#include "dom.h"
#include <iostream>

using namespace std;
using namespace xdom;

#ifndef PI
#define PI 3.1415926
#endif

Resources resources;

// adds all filenames that match pattern to the vector
void glob(const char *pattern, vector<string> &result)
{
	char buf[1024];
	
	al_ffblk info;
	
	int temp = al_findfirst (pattern, &info, FA_RDONLY | FA_ARCH);
	
	while (temp == 0)
	{
		replace_filename(buf, pattern, info.name, sizeof(buf));
		result.push_back (std::string(buf));
		temp = al_findnext(&info);
	}
			
	al_findclose(&info);
}

int Resources::init()
// returns 1 on succes, 0 on failure
{
	// load all .dat files in data directory
	vector <string> datafilenames;
	glob ("data/*.dat", datafilenames);
	for (vector<string>::iterator i = datafilenames.begin();
		i != datafilenames.end(); ++i)
	{
		DATAFILE *temp;
		if (!(temp = load_datafile (i->c_str())))
		{
			return 0;
		} else {
				data.push_back (temp);
		}
	}

	vector <string> bmpfilenames;	
	glob ("images/*.bmp", bmpfilenames);
	
	for (vector<string>::iterator i = bmpfilenames.begin();
		i != bmpfilenames.end(); ++i)
	{
		unsigned int start = i->find_last_of ("\\/");
		if (start == string::npos) start = 0;
		else start += 1;
			
		unsigned int end = i->rfind (".");
		if (end == string::npos) end = i->size();
		else end -= 1;
			
		BITMAP *bmp;
		bmp = load_bitmap (i->c_str(), NULL);
		if (!bmp)
		{
			return 0;
		}
		else
		{
			//~ cout << *i << " " << i->substr(start, end-start+1) << endl;
			sprites.insert (pair<string, RLE_SPRITE*>(i->substr(start, end-start+1), get_rle_sprite(bmp)));
			destroy_bitmap (bmp);
		}
	}
	
	// load xml data
	DomNode node = xmlParseFile("data/dialogs.xml");
	
	vector<DomNode>::iterator n;
	for (n = node.children.begin(); n != node.children.end(); ++n)
	{
		if (n->name == "dialog")
		{
			string name = n->attributes["name"];
			dialogs.insert (pair <string, DomNode>(name, (*n)));			
		}
	}	
	return 1;
}

Resources::~Resources()
{
	{
		map<string, RLE_SPRITE*>::iterator i;
		for (i = sprites.begin(); i != sprites.end(); i++)
		{
			destroy_rle_sprite (i->second);
		}
	}
	{
		vector<DATAFILE*>::iterator i;
		for (i = data.begin(); i != data.end(); ++i)
		{
				unload_datafile (*i);
				*i = NULL;
		}
	}
}

DomNode &Resources::getDialog (string id)
{
	assert (dialogs.find (id) != dialogs.end());
	return dialogs[id];	
}

FONT *Resources::getFont (string id)
{
	vector<DATAFILE*>::iterator i;
	DATAFILE *temp = NULL;
	for (i = data.begin(); i != data.end() && temp == NULL; ++i)
	{
			assert (*i);
			temp = find_datafile_object (*i, id.c_str());				
			// check if object is of the right type
			if (temp && temp->type != DAT_FONT)
					temp = NULL;
	}
	if (temp)
			return (FONT*)temp->dat;
	else
			return NULL;	
}

// look in datafile first, then look in individual bitmaps
RLE_SPRITE *Resources::getRle (string id)
{
	vector<DATAFILE*>::iterator i;
	DATAFILE *temp = NULL;
	for (i = data.begin(); i != data.end() && temp == NULL; ++i)
	{
			assert (*i);
			temp = find_datafile_object (*i, id.c_str());				
			// check if object is of the right type
			if (temp && temp->type != DAT_RLE_SPRITE)
					temp = NULL;
	}
	
	if (temp)
			return (RLE_SPRITE*)temp->dat;
	else
	{
		if (sprites.find (id) == sprites.end())
		{
			allegro_message ("Couldn't find RLE sprite '%s'", id.c_str());			
			exit(1);
		}
		
		return sprites[id];
	}
	
	assert (false);
	return NULL;
}

SAMPLE *Resources::getSample (string id)
{
	vector<DATAFILE*>::iterator i;
	DATAFILE *temp = NULL;
	for (i = data.begin(); i != data.end() && temp == NULL; ++i)
	{
			assert (*i);
			temp = find_datafile_object (*i, id.c_str());				
			// check if object is of the right type
			if (temp && temp->type != DAT_SAMPLE)
					temp = NULL;
	}
	if (temp)
			return (SAMPLE*)temp->dat;
	else
			return NULL;
}
